/*
 * test
 *
 * Copyright (c) 2015 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations uynder the License.
 */


#include "test.h"
#include <stdio.h>

#define METHOD_LED_PLAYCUSTOM				"playcustom"
#define METHOD_LED_STOPCUSTOM				"stopcustom"
//#define METHOD_LED_GETBRIGHTNESS_FORCAMERA	"GetBrightnessForCamera"
#define METHOD_LED_GETBRIGHTNESS			"GetBrightness"
#define METHOD_LED_GETMAXBRIGHTNESS			"GetMaxBrightness"
#define METHOD_LED_SETBRIGHTNESS			"SetBrightness"

#define RED 0xff0000
#define GREEN 0x00ff00
#define BLUE 0x0000ff
#define DEFAULT_FLAG	(1 << 0)

static bool get_led_method(const char *method)
{
	GVariant *msg;
	int val;
	bool ret = FALSE;

	msg = dbus_handle_method_sync_with_reply(DEVICED_BUS_NAME,
					DEVICED_PATH_LED,
					DEVICED_INTERFACE_LED,
					method, NULL, NULL);
	if (!msg) {
		_E("fail (%s): no reply", method);
		return ret;
	}

	if (!dh_get_param_from_var(msg, "(i)", &val))
		_E("fail (%s): no message", method);
	else {
		_I("success (%s): %d", method, val);
		ret = TRUE;
	}

	g_variant_unref(msg);
	return ret;
}

static bool set_led_method(const char *method, GVariant *param)
{
	GVariant *msg;
	int val;
	bool ret = FALSE;

	msg = dbus_handle_method_sync_with_reply_var(DEVICED_BUS_NAME,
						DEVICED_PATH_LED,
						DEVICED_INTERFACE_LED,
						method, param);
	if (!msg) {
		_E("fail (%s): no reply", method);
		return ret;
	}

	if (!dh_get_param_from_var(msg, "(i)", &val))
		_E("fail (%s): no message", method);
	else {
		if ((val == -ENOTSUP) || (val == -ENOSYS)) {
			_I("Not supported feature! (%s): %d", method, val);
			ret = TRUE;
		} else if (val < 0) {
			_E("fail (%s): returned fail (%d)", method, val);
		} else {
			_I("success (%s): %d", method, val);
			ret = TRUE;
		}
	}

	g_variant_unref(msg);
	return ret;
}

static bool get_led_maxbrightness(void)
{
	return get_led_method(METHOD_LED_GETMAXBRIGHTNESS);
}

static bool get_led_brightness(void)
{
	return get_led_method(METHOD_LED_GETBRIGHTNESS);
}

static bool set_led_brightness(int brightness, int enabled)
{
	return set_led_method(METHOD_LED_SETBRIGHTNESS, g_variant_new("(ii)", brightness, enabled));
}

static bool set_led_playcustom(int on, int off, unsigned int color, unsigned int flags)
{
	return set_led_method(METHOD_LED_PLAYCUSTOM, g_variant_new("(iiuu)", on, off, color, flags));
}

static bool set_led_stopcustom()
{
	return set_led_method(METHOD_LED_STOPCUSTOM, NULL);
}

void led_test_all(int *success, int *fail)
{
	int s = 0;
	int f = 0;

	(get_led_maxbrightness())		? s++ : f++;
	(set_led_brightness(90, 1))		? s++ : f++;
	(set_led_brightness(0, 1))		? s++ : f++;
	(get_led_brightness())			? s++ : f++;
	(set_led_playcustom(1, 0, RED, DEFAULT_FLAG))	? s++ : f++;
	(set_led_stopcustom())			? s++ : f++;

	if (NULL != success)    *success = s;
	if (NULL != fail)       *fail = f;
}

static void led_init(void *data)
{
	int success = 0;
	int fail = 0;

	_I("start test");

	led_test_all(&success, &fail);

	_I("Total: %d, Success: %d, Fail: %d", success+fail, success, fail);
}

static void led_exit(void *data)
{
	_I("end test");
}

static int led_unit(int argc, char **argv)
{
	if (argc < 4) {
		int success = 0;
		int fail = 0;
		_I("start test");
		led_test_all(&success, &fail);
		_I("Total: %d, Success: %d, Fail: %d", success+fail, success, fail);
	} else if (0 == strcasecmp(argv[3], METHOD_LED_GETMAXBRIGHTNESS)) {
		get_led_maxbrightness();
	} else if (0 == strcasecmp(argv[3], METHOD_LED_GETBRIGHTNESS)) {
		get_led_brightness();
	} else if (0 == strcasecmp(argv[3], METHOD_LED_SETBRIGHTNESS)) {
		set_led_brightness(atoi(argv[4]), atoi(argv[5]));
	} else if (0 == strcasecmp(argv[3], METHOD_LED_PLAYCUSTOM)) {
		set_led_playcustom(atoi(argv[4]), atoi(argv[5]), strtoul(argv[6], NULL, 16), DEFAULT_FLAG);
	} else if (0 == strcasecmp(argv[3], METHOD_LED_STOPCUSTOM)) {
		set_led_stopcustom();
	} else {
		_E("Unknown test case!!!");
	}

	return 0;
}

static const struct test_ops led_test_ops = {
	.priority = TEST_PRIORITY_NORMAL,
	.name     = "led",
	.init     = led_init,
	.exit     = led_exit,
	.unit     = led_unit,
};

TEST_OPS_REGISTER(&led_test_ops)
