/*
 * deviced
 *
 * Copyright (c) 2014 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *	 http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <assert.h>
#include <gio/gio.h>
#include <dlfcn.h>

#include "common.h"
#include "core/log.h"

#ifndef LIBPATH
#error LIBPATH is not defined.
#endif
#define MODULE_PATH		LIBPATH"/deviced"

int load_plugin(const char *id, void **h)
{
	char path[PATH_MAX];
	void *handle;

	if (!id || !h)
		return -EINVAL;

	/* Find matched module path */
	snprintf(path, sizeof(path), "%s/%s.so", MODULE_PATH, id);
	if (access(path, R_OK) != 0) {
		_E("there is no %s device", id);
		return -ENODEV;
	}

	/* Load module */
	handle = dlopen(path, RTLD_LAZY|RTLD_GLOBAL);
	//handle = dlopen(path, RTLD_NOW|RTLD_GLOBAL);
	if (!handle) {
		_E("fail to open module : %s", dlerror());
		return -ENOENT;
	}

	*h = handle;

	return 0;
}


int unload_plugin(void *h)
{
	if (!h)
		return 0;
	else
		return dlclose(h);
}
